<?php declare(strict_types=1);

namespace Everyware\Widget\ArticleList;

use Infomaker\Everyware\Base\OpenContent\QueryBuilder;
use Infomaker\Everyware\Base\Pages;
use Infomaker\Everyware\Base\Templates;
use Infomaker\Everyware\Support\Str;
use Infomaker\Everyware\Twig\View;
use Infomaker\ProjectPlugin\Components\SettingsHandler;
use Infomaker\ProjectPlugin\Components\Widget;
use Infomaker\ProjectPlugin\Helpers\FileReader;
use OcAPI;

class Component extends Widget
{
    protected static $fields = [
        'headline' => '',
        'page' => '',
        'template' => '',
        'remove_list_elements' => false,
        'oc_query' => '',
        'oc_query_start' => 0,
        'oc_query_limit' => 10,
        'oc_query_sort' => '',
    ];

    /**
     * Contains the API-instance to connect to Open Content
     *
     * @var OcAPI
     */
    protected $ocApi;

    /**
     * @var string
     */
    private $templatePath = '@widgets/everyware-widget-article-list/src/templates';

    /**
     * @var string
     */
    private $defaultTemplate = 'ocarticle-default';

    public function __construct(SettingsHandler $settingsHandler, FileReader $fileReader)
    {
        $this->ocApi = new OcAPI();
        parent::__construct($settingsHandler, $fileReader);
    }

    /**
     * Function to get available sort options
     *
     * @return array
     */
    protected function sortingsSelectData(): array
    {
        $sort_options = $this->ocApi->get_oc_sort_options();

        if (isset($sort_options->sortings)) {
            return array_map(function ($option) {
                return [
                    'value' => $option->name,
                    'text' => $option->name
                ];
            }, $sort_options->sortings);
        }

        return [];
    }

    /**
     * Function to get available properties from a contenttype as select-data
     *
     * @param string $contenttype
     *
     * @since 0.1
     * @return array
     */
    protected function propertiesSelectData($contenttype = 'Article'): array
    {
        $query_options = $this->ocApi->get_content_types();

        $properties = [];
        foreach ($query_options[$contenttype] as $property) {
            $property = $property[0] ?? '';
            if ( ! Str::isEmpty($property)) {
                $properties[] = [
                    'value' => $property,
                    'text' => Str::ucfirst($property)
                ];
            }
        }

        return $properties;
    }

    /**
     * Function to get available contenttypes options
     *
     * @since 0.1
     * @return array
     */
    protected function contenttypesSelectData(): array
    {
        return array_map(function ($contenttype) {
            return [
                'value' => $contenttype,
                'text' => $contenttype
            ];
        }, array_keys($this->ocApi->get_content_types()));
    }

    /**
     * Function to fetch the articles from Open Content
     *
     * @param $viewData
     *
     * @return array
     */
    private function fetchArticles($viewData): array
    {
        $this->ocApi->prepare_widget_query($this->convertWidgetQuery($viewData));

        return $this->generateFromTemplate($this->ocApi->get_oc_articles(), $viewData['template']);
    }

    /**
     * Run the article through the template to generate HTML
     *
     * @param \OcArticle[] $ocArticles
     * @param string       $template
     *
     * @return array
     */
    private function generateFromTemplate($ocArticles, $template = ''): array
    {
        $templateFile = Str::notEmpty($template) ? $template : $this->defaultTemplate;
        $template = locate_template([$templateFile, "/templates/{$templateFile}"]);

        $articles = [];
        ob_start();
        foreach ($ocArticles as $index => $article) {
            include $template;
            $articles[] = ob_get_contents();
            ob_clean();
        }
        ob_end_clean();

        return $articles;
    }

    /**
     * Convert the form-data into values readable by OpenContent::search()
     *
     * @param $viewData
     *
     * @return array
     */
    protected function convertWidgetQuery($viewData): array
    {
        $query = $viewData['oc_query'];

        if ( ! Str::contains($query, 'contenttype:Article')) {
            $viewData['oc_query'] = QueryBuilder::query($query)->prepend('contenttype:Article')->getQuery();
        }

        return $viewData;
    }

    public function getOcQuery($storedData): string
    {
        if (isset($storedData)) {
            $this->ocApi->prepare_widget_query($storedData);
        }

        return $this->ocApi->get_active_query();
    }

    public function form(array $viewData): string
    {
        $viewData = array_replace_recursive($viewData, [
            'pages' => Pages::getSelectData(),
            'templates' => Templates::getSelectData(),
            'sortings' => $this->sortingsSelectData(),
            'settings' => $this->getFormSettings()
        ]);

        return View::generate($this->templatePath . '/form', $viewData);
    }

    public function render(array $viewData): string
    {
        $viewData = array_replace_recursive($viewData, [
            'articles' => $this->fetchArticles($viewData)
        ]);

        return View::generate($this->templatePath . '/widget', $viewData);
    }

    public static function setup($file): void
    {
        $fileReader = new FileReader($file);

        $headerData = $fileReader->extractHeaders([
            'name' => 'Plugin Name',
            'version' => 'Version'
        ]);

        $settingsHandler = new SettingsHandler(
            'ew-' . Str::slug($headerData['name']),
            static::$fields,
            $headerData['version']
        );

        $component = new static($settingsHandler, $fileReader);

        $component->register();
    }
}
